@@ -14,7 +14,7 @@ from motorcortex.setup_logger import logger
 from motorcortex.state_callback_handler import StateCallbackHandler
 
 import os
-import queue
+from threading import Condition  # Add this import
 from concurrent.futures import ThreadPoolExecutor
 from pynng import Req0, TLSConfig
 from enum import Enum
@@ -28,32 +28,12 @@ class ConnectionState(Enum):
     DISCONNECTING = 4
     DISCONNECTED = 5
 
-
-class Base64Encoder(json.JSONEncoder):
-    # pylint: disable=method-hidden
-    encoding = 'utf-8'
-
-    def default(self, o):
-        if isinstance(o, bytes):
-            return base64.b64encode(o).decode(Base64Encoder.encoding)
-        return json.JSONEncoder.default(self, o)
-
-
-class Base64Decoder(json.JSONDecoder):
-    # pylint: disable=method-hidden
-    encoding = 'utf-8'
-
-    def default(self, o):
-        if isinstance(o, bytes):
-            return base64.b64decode(o).encode(Base64Decoder.decode())
-        return json.Base64Decoder.default(self, o)
-
-
 class Request(object):
 
     def __init__(self, protobuf_types, parameter_tree):
         self.__socket = None
         self.__url = None
+        self.__connected = False
         self.__connected_lock = None
         self.__protobuf_types = protobuf_types
         self.__parameter_tree = parameter_tree
@@ -77,22 +57,27 @@ class Request(object):
             tls_config = TLSConfig(TLSConfig.MODE_CLIENT, ca_files=certificate)
 
         self.__socket = Req0(recv_timeout=recv_timeout_ms, tls_config=tls_config)
-        self.__connected_lock = queue.Queue()
+        self.__connected_lock = Condition()  # Create Condition instead of Queue
+        self.__connected = False  # Initialize state
 
-        def pre_connect_cb(pipe):
-            self.__connected_lock.put(True)
-            self.__connection_state = ConnectionState.CONNECTION_OK
-            self.__callback_handler.notify(self, self.connectionState())
+        def pre_connect_cb(_pipe):
+            with self.__connected_lock:
+                self.__connected = True
+                self.__connection_state = ConnectionState.CONNECTION_OK
+                self.__callback_handler.notify(self, self.connectionState())
+                self.__connected_lock.notify_all()  # Wake up all waiting threads
 
-        def post_remove_cb(pipe):
-            if self.__connection_state == ConnectionState.DISCONNECTING:
-                self.__connection_state = ConnectionState.DISCONNECTED
-            elif self.__connection_state == ConnectionState.CONNECTING:
-                self.__connection_state = ConnectionState.CONNECTION_FAILED
-            elif self.__connection_state == ConnectionState.CONNECTION_OK:
-                self.__connection_state = ConnectionState.CONNECTION_LOST
-            self.__connected_lock.put(False)
-            self.__callback_handler.notify(self, self.connectionState())
+        def post_remove_cb(_pipe):
+            with self.__connected_lock:
+                if self.__connection_state == ConnectionState.DISCONNECTING:
+                    self.__connection_state = ConnectionState.DISCONNECTED
+                elif self.__connection_state == ConnectionState.CONNECTING:
+                    self.__connection_state = ConnectionState.CONNECTION_FAILED
+                elif self.__connection_state == ConnectionState.CONNECTION_OK:
+                    self.__connection_state = ConnectionState.CONNECTION_LOST
+                self.__connected = False
+                self.__callback_handler.notify(self, self.connectionState())
+                self.__connected_lock.notify_all()  # Wake up all waiting threads
 
         self.__socket.add_post_pipe_connect_cb(pre_connect_cb)
         self.__socket.add_post_pipe_remove_cb(post_remove_cb)
@@ -104,9 +89,12 @@ class Request(object):
     def close(self):
         self.__connection_state = ConnectionState.DISCONNECTING
         if self.__connected_lock:
-            self.__connected_lock.put(False)
+            with self.__connected_lock:
+                self.__connected = False
+                self.__connected_lock.notify_all()
         self.__socket.close()
         self.__callback_handler.stop()
+        self.__pool.shutdown(wait=True)
 
     def send(self, encoded_msg, do_not_decode_reply=False):
         if self.__socket is not None:
@@ -146,7 +134,7 @@ class Request(object):
         """Request a parameter tree hash from the server.
 
             Returns:
-                Reply(ParameterTreeMsg): A Promise, which resolves when parameter tree hash is received or fails
+                Reply(ParameterTreeMsg): A Promise, which resolves when a parameter tree hash is received or fails
                 otherwise. ParameterTreeHashMsg message has a status field to check the status of the operation.
 
             Examples:
@@ -179,14 +167,14 @@ class Request(object):
                                         self.getParameterTreeHash(), self.__protobuf_types, self.__socket))
 
     def save(self, path, file_name):
-        """Request a server to save a parameter tree to file.
+        """Request a server to save a parameter tree to a file.
 
             Args:
                 path(str): path where to save
                 file_name(str): file name
 
             Returns:
-                Reply(StatusMsg): A promise, which resolves when save operation is completed,
+                Reply(StatusMsg): A promise, which resolves when the save operation is completed,
                 fails otherwise.
 
         """
@@ -209,7 +197,7 @@ class Request(object):
                 value argument)
 
             Returns:
-                  Reply(StatusMsg): A Promise, which resolves when parameter value is updated or fails otherwise.
+                  Reply(StatusMsg): A Promise, which resolves when a parameter value is updated or fails otherwise.
 
             Examples:
                   >>> reply = req.setParameter("root/Control/activateSemiAuto", False)
@@ -499,13 +487,17 @@ class Request(object):
         return None
 
     @staticmethod
-    def waitForConnection(connected_lock, timeout_sec):
-        if timeout_sec <= 0:
-            timeout_sec = None
-        try:
-            return connected_lock.get(block=True, timeout=timeout_sec)
-        except queue.Empty:
-            return False
+    def waitForConnection(condition, timeout_sec):
+        with condition:
+            # Wait for the condition to be set or timeout
+            if timeout_sec <= 0:
+                result = condition.wait()
+            else:
+                result = condition.wait(timeout_sec)
+
+            if not result:
+                raise TimeoutError("Connection timeout")
+            return True
 
     @staticmethod
     def __getParameterTree(hash_reply, protobuf_types, socket):
